#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Super Class for all commands
#
# Copyright(C) 2005-, Atzm WATANABE <sitosito@p.chan.ne.jp>
#
# $Id: $

import shlex, os, fcntl, sys
from RSS       import TrackingChannel
from scheduler import Scheduler
from common    import get_all_nick, unicoding, ununicoding, ns
from htmlizer  import HTMLizer

RSS_MAIN = '/./{[|main|]}/./'
class SuperCommand(Scheduler):
	def __init__(self, config, **positions):
		"""
		super class for all commands:
			positions: each arguments' position, ex. command=0, url=1, title=2, ...

		scheduled terms in SuperCommand:
			1. analyze
				1.1. split
			2. modify
				2.1. load_rss_for_multi

		avaliable procs:
			split:              analyzes and splits command's argment
			load_rss_for_multi: loads RSS file
			write_to_rss:       write out to RSS file
			write_to_html:      write out to HTML file

		avaliable terms:
			after_write_term:   notice that wrote out to file
		"""

		if not positions:
			self._args_num  = 0
		else:
			self._args_num  = max(positions.values())

		self._command_args  = {}
		self._positions     = positions
		self._config        = config

		self._htmlizer      = HTMLizer(self._config)
		self._rss           = {}
		self._rss[RSS_MAIN] = TrackingChannel()
		self.load_rss()

		if self._config.INDIVIDUAL_MEMO:
			for nick in get_all_nick(self._config.OUTPUT_RSS_PATH):
				self._rss[nick] = self.get_rss_individual(nick)

		Scheduler.__init__(self)
		Scheduler.append_term(self, 'analyze', ['split'])
		Scheduler.append_term(self, 'modify',  ['load_rss_for_multi'])

# -- Utilities for around split proc
	def get_arg(self, name):
		"""get a command argument named 'name'"""
		name = str(name)
		return self._command_args.get(name, '')

	def set_arg(self, name, value):
		"""set a command argument named 'name' to 'value'"""
		name = str(name)
		self._command_args[name] = value

# -- Available procs
	def _split(self, c, e, msg, nick, channel):
		"""splits command line args"""
		try:
			command_args = shlex.split(ununicoding(msg))
			if len(command_args) != self._args_num+1: raise ValueError
			self._command_args = {}
			for name, pos in self._positions.items():
				if pos is None: continue
				self._command_args[name] = unicoding(command_args[pos])
		except:
			return False
		return True

	def _load_rss_for_multi(self, c, e, msg, nick, channel):
		"""load rss for parallel process"""
		try:    self.load_rss(nick)
		except: return False
		return True

	def _write_to_rss(self, c, e, msg, nick, channel):
		"""write out to rss"""
		try:    self.write_rss(nick)
		except: return False
		return True

	def _write_to_html(self, c, e, msg, nick, channel):
		"""write out to html"""
		try:    self.write_html(nick)
		except: return False
		return True

	def _after_write_term(self, c, e, msg, nick, channel):
		"""is called after all process has done"""
		try:    c.notice(nick, 'Wrote out successfully.')
		except: return False
		return True

# --- Other utilities
	def inspect_nick(self, nick):
		"""get inspected nickname"""
		if nick is None:
			return None

		nick = str(nick).replace('.', '').replace('/', '').rstrip(self._config.STRIP_NICK)

		if not nick:
			return None

		return nick

	def get_rss_path_from_nick(self, nick):
		"""get individual output RSS path (absolute) from nick"""
		nick = self.inspect_nick(nick)
		assert nick is not None, 'nick is None'
		return os.path.abspath(os.path.join(os.path.dirname(self._config.OUTPUT_RSS_PATH), '%s.rdf' % nick))

	def get_html_path_from_nick(self, nick):
		"""get individual output HTML path (absolute) from nick"""
		nick = self.inspect_nick(nick)
		assert nick is not None, 'nick is None'
		return os.path.abspath(os.path.join(os.path.dirname(self._config.OUTPUT_RSS_PATH), '%s.html' % nick))

	def get_rss_individual(self, nick):
		"""get nick's RSS obj"""
		nick = self.inspect_nick(nick)
		assert nick is not None, 'nick is None'

		try:
			return self._rss[nick]
		except KeyError:
			pass

		rss_path = self.get_rss_path_from_nick(nick)
		if rss_path == self._config.OUTPUT_RSS_PATH:
			sys.stderr.write("!! Warning: main RSS, not individual: %s\n" % nick)
			return None

		self._rss[nick] = TrackingChannel()
		if os.path.isfile(rss_path):
			self._rss[nick].parse(rss_path)
		else:
			self._rss[nick].parse(self._config.RSS_TEMPLATE)
		return self._rss[nick]

	def load_rss(self, nick=None):
		"""load rss file"""
		nick = self.inspect_nick(nick)

		targets = [(self._rss[RSS_MAIN], self._config.OUTPUT_RSS_PATH)]
		if self._config.INDIVIDUAL_MEMO and nick is not None:
			targets.append((self.get_rss_individual(nick), self.get_rss_path_from_nick(nick)))

		for rss, rss_path in targets:
			self.__clear_rss(rss)
			if os.path.exists(rss_path):
				rss.parse(rss_path)

	def __clear_rss(self, rss):
		"""clear RSS obj (internal)"""
		rss.clear_rss()
		if os.path.exists(self._config.RSS_TEMPLATE):
			rss.parse(self._config.RSS_TEMPLATE)

	def add_item(self, url, title, description, date, nick=None):
		"""add item to rss"""
		nick = self.inspect_nick(nick)

		targets = [self._rss[RSS_MAIN]]
		if self._config.INDIVIDUAL_MEMO and nick is not None:
			targets.append(self.get_rss_individual(nick))

		for rss in targets:
			rss.addItem({
				(ns.rss10, 'link'):        url,
				(ns.rss10, 'title'):       title,
				(ns.rss10, 'description'): description,
				(ns.dc,    'date'):        date,
				})

	def write_rss(self, nick=None):
		"""write out rss file"""
		nick = self.inspect_nick(nick)

		targets = [(self._rss[RSS_MAIN], None)]
		if self._config.INDIVIDUAL_MEMO and nick is not None:
			targets.append((self.get_rss_individual(nick), nick))

		for rss, nick in targets:
			if self._config.SAVE_LIMIT <= 0:
				items = rss.listItems()
			else:
				items = rss.listItems()[:self._config.SAVE_LIMIT]

			if nick is None:
				path = self._config.OUTPUT_RSS_PATH
			else:
				path = self.get_rss_path_from_nick(nick)

			f = file(path, 'w+')
			fcntl.flock(f.fileno(), fcntl.LOCK_EX)
			f.write(rss.output(items))
			fcntl.flock(f.fileno(), fcntl.LOCK_UN)
			f.close()

			if self._config.SAVE_LIMIT > 0:
				self.load_rss(nick)

	def write_html(self, nick=None):
		"""write out html file"""
		nick = self.inspect_nick(nick)

		# [(RSSobj, rss_place, page_title, author_name, html_path)]
		targets = [(
			self._rss[RSS_MAIN],
			self._config.RSS_PLACE,
			self._config.PAGE_TITLE,
			self._config.AUTHOR_NAME,
			self._config.OUTPUT_HTML_PATH,
			)]

		if self._config.INDIVIDUAL_MEMO and nick is not None:
			targets.append((
				self.get_rss_individual(nick),
				os.path.join(os.path.dirname(self._config.RSS_PLACE), '%s.rdf' % nick),
				"%s's %s" % (nick, self._config.PAGE_TITLE),
				nick,
				self.get_html_path_from_nick(nick),
				))

		for rss, rss_place, page_title, author_name, html_path in targets:
			self._htmlizer.write(rss, rss_place, page_title, author_name, html_path)

	# FIXME: can't keep consistency when individual memo available!
	def remove_prev_item(self, nick=None):
		"""remove previous rss entry"""
		nick = self.inspect_nick(nick)

		targets = [self._rss[RSS_MAIN]]
		if self._config.INDIVIDUAL_MEMO and nick is not None:
			targets.append(self.get_rss_individual(nick))

		for rss in targets:
			rss.removeItem((self.get_previous_uri(rss), 0))

	def get_previous_uri(self, rss=None):
		"""get previous rss entry"""
		if rss is None: rss = self._rss[RSS_MAIN]
		return rss.data[(ns.rss10, 'items')][0][0]

	def load_config(self):
		"""load config file"""
		self._config.load()
